function [output_sequence, appearance] = arithmetic_encoder(input_sequence, width)
% Syntax
% INPUT:
% - input_sequence ... input text
% - width ... length of chunks
% OUTPUT:
% - output_sequence ... serie of real numbers
% - appearance ... probability of appearance of individual symbols in input
%                  sequence

output_sequence = [];
appearance = [];

if (rem(length(input_sequence),width) ~= 0)
    fprintf('Length of the input sequence should be multiple of "width"!\n');
    return;
end
   
% convert the array from characters to numbers
input = uint8(input_sequence);

% create the table of symbol appearance and normalize it
appearance = zeros(1,256);

for it=1:length(input)
    current_value = input(it);
    appearance(current_value) = appearance(current_value) + 1;
end

appearance = appearance / length(input);

% sort the table
[table, indices] = sort(appearance,2,'descend');

% make the table cumulative
for it=2:256
    table(it) = table(it)+table(it-1);
end

lower_limit = 0.0;
upper_limit = 1.0;
curr_len = 0;

while (~isempty(input))
    % read the current symbol
    symbol = input(1);
    
    % update the length of already read sequence
    curr_len = curr_len + 1;
           
    % shorten the input sequence
    input = input(2:end);
    
    % find out the absolute location of currently inspected symbol
    Upper = table(find(indices==symbol));
    Lower = Upper - appearance(symbol);
    
    % make this location relative
    Upper = Upper*(upper_limit-lower_limit) + lower_limit;
    Lower = Lower*(upper_limit-lower_limit) + lower_limit;
  
    % update the limits
    upper_limit = Upper;
    lower_limit = Lower;
        
    if ((curr_len == width) || (isempty(input)))
        % at the end of the sequence or reached allowed width of the
        % procesed sequence of symbols?
        output_sequence = [output_sequence (lower_limit+upper_limit)/2];
            
        % restart the algorithm
        curr_len = 0;
        lower_limit = 0.0;
        upper_limit = 1.0;        
    end
end

end
   
   
        
        
    
    


